<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsCartRule.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsCartRuleImport
{
    /**
     * Voucher import: fields list for new voucher
     * used in importCartRuleFromCsv()
     */
    public static $definitions = [
        'field_key' => [
            'cart_rule_code',       // = null, used as prefix if random_code is true
            'random_code',          // = false, bool: true=generate a random code with cart_rule_code as prefix
            'code_len',             // = 12, length of code (max 254 chars)
            'quantity_to_create',   // = 1, quantity of rules to create
            'quantity',             // = 1, quantity of single cart rule
            'quantity_per_user',    // = 1, quantity of single cart rule per user
            'priority',             // = 1 (priority of the voucher on other vouchers)
            'cart_rule_name',       // = null,  name of cart rule
                                        // String: 1 language (an array will be created)
                                        // Array for any language
                                        // if empty: name will be cart_rule_code
            'cart_rule_description', // = '', string
            'id_customer',          // = 0, restrict to one customer
            'group_ids',            // = null, array of groups for group restrinction.
                                        // Can be also a string with ID separated by comma
            'category_ids',         // = null, array of categories for category restrinction.
                                        // Can be also a string with ID separated by comma
            'partial_use',          // = 0, can cart rule be used partially ?
            'highlight',            // = 0, does the voucher be listed in user cart voucher list ?
            'free_shipping',        // = 0, 1 = free
            'reduction_percent',    // = 0, your percentage or 0.00;
            'reduction_amount',     // = 0, your_amount or 0.00;
            'reduction_tax',        // = 0, 1 = tax included;
            'date_from',            // = null, start date / time = date('Y-m-d H:i:s');
            'date_to',              // = null end date / time = date('Y-m-d H:i:s');
            // 'unique_for_customer',// = null if 1, the customer cannot have more than 1 voucher.
                                        // Check existing vouchers and skip or save this one
            'check_active',         // if true voucher will be created only if customer has no other "active" vouchers
            'check_inStock',        // if true voucher will be created only if customer has no other "inStock" vouchers
            'check_includeGeneric', // if true voucher will be created only if customer has no other "generic" vouchers
                                    // (automatic, without a target user and with or without a code to insert)
        ],
        'target_folder' => _PS_MODULE_DIR_ . 'mxb_tools' . DIRECTORY_SEPARATOR . 'csv' . DIRECTORY_SEPARATOR,
        'target_filename' => 'mxb_cart_rules_import.csv',
        'sample_filename' => 'mxb_cart_rules_import_sample.csv',
        'csv_delimeter' => ';',
    ];

    public static function importCartRuleFromCsv($limit = 0, $id = 0, $tot_records = 0)
    {
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsCartRuleImport::importCartRuleFromCsv',
            'id=' . $id . ', limit=' . $limit
        );
        */
        // NO DEBUG: set $simulate to false !
        $simulate = false;
        $errors = [];
        $success = [];

        $csv = MxbToolsUtils::csvToArray(
            self::$definitions['target_folder'] . self::$definitions['target_filename'],
            self::$definitions['csv_delimeter']
        );
        if (!$csv) {
            return [
                'records' => -1,
                'tot_records' => 0,
                'id' => 0,
                'errors' => MxbToolsCartRuleImport::addImportLog('-1', 'CSV FILE', 'error converting CSV file.'),
            ];
        }

        // don't accept single record
        // if ($limit === 1)
        //  $limit = 2;
        // if ($limit > 0 && $id === 0) {
        if ((int) $tot_records === 0) {
            $tot_records = count($csv); // Db::getInstance(_PS_USE_SQL_SLAVE_)->getValue($sql);
            // } else {
            //    $tot_records = 0;
        }
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsCartRuleImport::importCartRuleFromCsv',
            'tot_records=' . $tot_records
        );
        */
        $next_id = (int) $id + 1;
        $inserted_records = 0;
        $write_mode = 1; // create csv

        // import main loop
        foreach ($csv as $rowkey => $row) {
            set_time_limit(300);

            // skip to current position
            if ((int) $rowkey < (int) $id) {
                $write_mode = 2; // append to csv
                /*
                MxbToolsDebug::writeLog(
                    'Mxb_Tools',
                    'MxbToolsCartRuleImport::importCartRuleFromCsv',
                    'LOOP ' . $rowkey . ' => continue'
                );
                */
                continue;
            }
            $added = false;
            $next_id = (int) $rowkey + 1;

            /*
              CSV file must have every field in the right position because values will be take by its field position:
                0 cart_rule_code (empty / voucher code / suffix for random)
                1 random_code (0/1: 1=generate a random code with cart_rule_code as prefix)
                2 code_len (for rnd generation)
                3 quantity_to_create (quantity of rules to create)
                4 quantity (quantity of single cart rule)
                5 quantity_per_user (quantity of single cart rule per user)
                6 priority (priority of the voucher on other vouchers)
                7 cart_rule_name
                8 cart_rule_description
                9 id_customer (restrict to one customer)
               10 group_ids (a string with group ID separated by comma)
               11 category_ids (a string with category ID separated by comma)
               12 partial_use (0/1 - can cart rule be used partially ?)
               13 highlight (0/1 - can cart rule be listed in user cart ?)
               14 free_shipping (0/1)
               15 reduction_percent (your percentage or 0)
               16 reduction_amount (your_amount or 0)
               17 reduction_tax (0/1 - 1 = tax included)
               18 date_from (yyyy-mm-dd hh:mm:ss)
               19 date_to (yyyy-mm-dd hh:mm:ss)
               20 unique_for_customer (if true, voucher will be created only if the customer has no other vouchers)
            */

            $i = 0;
            $cartRuleData = [];
            foreach ($row as $c) {
                $cartRuleData[self::$definitions['field_key'][$i]] = $c;
                ++$i;
            }
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsCartRuleImport::importCartRuleFromCsv',
                'LOOP ' . $rowkey . ' ARRAY customerData=' . print_r($cartRuleData, true)
            );
            */
            // look for errors
            /*
            if ($customerData['email'] == '') {
                $errors[] = MxbToolsCustomerImport::addImportLog($rowkey, '', 'missing email !!!');
                continue; // jump this customer
            }
            */
            // check data and correct
            /*
            if ($customerData['firstname'] == '' || $customerData['lastname'] == '') {
                $errors[] = MxbmpCustomerImport::addImportLog(
                    $rowkey, $customerData['email'],
                    'missing first or last name.'
                );
                continue; // jump this customer
            }
            */
            // set as [novalue] to tell the front JS to hide name and ask to for compile the field
            /*
            if ($customerData['firstname'] == '')
                $customerData['firstname'] = 'novalue';
            if ($customerData['lastname'] == '')
                $customerData['lastname'] = 'novalue';
            if ($customerData['company'] == '')
                $customerData['company'] = 'novalue';
            */

            // create the cart rule
            if (!$simulate) {
                $result = MxbToolsCartRule::createCartRule(
                    $cartRuleData['cart_rule_code'],    // used as prefix if random_code is true
                    (bool) $cartRuleData['random_code'], // true: generate random code with "cart_rule_code" as prefix
                    $cartRuleData['code_len'],          // length of code (max 254 chars)
                    $cartRuleData['quantity_to_create'], // quantity of rules to create
                    $cartRuleData['quantity'],          // quantity of single cart rule
                    $cartRuleData['quantity_per_user'], // quantity of single cart rule per user
                    $cartRuleData['priority'],          // priority of the voucher on other vouchers
                    $cartRuleData['cart_rule_name'],    // name of cart rule (array or string)
                    //                                      if empty: name will be cart_rule_code
                    $cartRuleData['cart_rule_description'], // string
                    $cartRuleData['id_customer'],       // id_customer: restrict to one customer
                    $cartRuleData['group_ids'],         // array of groups for group restrinction / ID string
                    $cartRuleData['category_ids'],      // array of categories for category restrinction / ID string
                    $cartRuleData['partial_use'],       // can cart rule be used partially ?
                    $cartRuleData['highlight'],         // can cart rule be listed in user cart ?
                    $cartRuleData['free_shipping'],     // 1 = free
                    $cartRuleData['reduction_percent'], // your percentage or 0.00;
                    $cartRuleData['reduction_amount'],  // your_amount or 0.00;
                    $cartRuleData['reduction_tax'],     // 1 = tax included;
                    $cartRuleData['date_from'],         // start date / time = date('Y-m-d H:i:s');
                    $cartRuleData['date_to'],           // end date / time = date('Y-m-d H:i:s'););
                    true,                               // $overwrite: true existing voucher code will be overwritten
                    $write_mode,                        // $create_csv: 1 = create new / 2 = append
                    (bool) $cartRuleData['check_active'],
                    (bool) $cartRuleData['check_inStock'],
                    (bool) $cartRuleData['check_includeGeneric']
                );
                $added = true;
                if ((int) $result['records'] > 0 || !isset($result['errors'])) {
                    $success[] = MxbToolsCartRuleImport::addImportLog(
                        $rowkey,
                        $cartRuleData['cart_rule_name'],
                        'cart rule(s) created: ' . $result['records']
                    );
                }
                /* TODO: find a way to join every csv row for every ajax call to createCartRule */
                if (isset($result['download_link']) && $rowkey >= count($csv) - 1) {
                    $success[] = 'Cart rules created! CSV file is ready to download! Click here: '
                        . '<a href="' . $result['download_url']
                        . '" target="_new">' . $result['download_url']
                        . '</a>';
                }
                if (isset($result['errors'])) {
                    if ((int) $result['records'] === 0) {
                        foreach ($result['errors'] as &$err) {
                            $err = MxbToolsCartRuleImport::addImportLog(
                                $rowkey,
                                $cartRuleData['cart_rule_name'],
                                'cart rule(s) created: ' . $result['records'] . ': ' . $err
                            );
                        }
                    }
                    $errors = array_merge($errors, $result['errors']);
                }
            } else {
                $added = true; // debug simulate
            }

            if ($write_mode === 1) {
                $write_mode = 2; // append to csv
            }
            if ($added && $rowkey < count($csv) - 1) {
                ++$inserted_records;
            }
            if ($limit > 0 && $inserted_records >= $limit) {
                break;
            }
        } // foreach ($csv as $rowkey=>$row)

        $ret = [
            'records' => $inserted_records,
            'tot_records' => $tot_records,
            'id' => $next_id,
        ];
        if (count($errors)) {
            $ret['errors'] = implode('|', $errors);
        }
        if (count($success)) {
            $ret['success'] = implode('|', $success);
        }

        return $ret;
    }

    public static function addImportLog($rowkey = 0, $code = '', $message = '')
    {
        return 'row ' . $rowkey . ' [' . $code . '] ' . $message;
    }

    public static function uploadCsvFile()
    {
        $source_fileinput = 'mxb_tools-upload-csv-cart_rule';
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsCartRuleImport::uploadCsvFile()',
            'mxb_tools-upload-csv-cart_rule-name='.Tools::getValue('mxb_tools-upload-csv-cart_rule-name')
        );
        */
        if (isset($_FILES[$source_fileinput])
            && isset($_FILES[$source_fileinput]['tmp_name'])
            && !empty($_FILES[$source_fileinput]['tmp_name'])
        ) {
            if (!move_uploaded_file(
                $_FILES[$source_fileinput]['tmp_name'],
                self::$definitions['target_folder'] . self::$definitions['target_filename']
            )) {
                return false;
            }
            return true;
        }

        return false;
    }
}
