<?php
/**
 * 2013-2024 MXB Informatica.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to info@mxbinformatica.it so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    MXB Informatica <info@mxbinformatica.it>
 *  @copyright 2013-2024 MXB Informatica
 *  @license   http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
 */

require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsDebug.php';
require_once _PS_MODULE_DIR_ . 'mxb_tools/classes/MxbToolsUtils.php';

if (!defined('_PS_VERSION_')) {
    exit;
}

class MxbToolsTextureImport // extends ObjectModel
{
    /**
     * texture import: fields list for textures
     * used in importFromCsv()
     */
    public static $definitions = [
        'field_key' => [
            'name' => 'name',
            'filename' => 'filename',
            'hex_colors' => 'hex_colors',
        ],
        'texture_format' => 'jpg',
        'target_folder' => _PS_MODULE_DIR_ . 'mxb_tools' . DIRECTORY_SEPARATOR . 'csv' . DIRECTORY_SEPARATOR,
        'target_filename' => 'mxb_texture_import.csv',
        'sample_filename' => 'mxb_texture_import_sample.csv',
        'csv_delimeter' => ';',
        'texture_folder' => _PS_MODULE_DIR_ . 'mxb_tools' . DIRECTORY_SEPARATOR . 'textures' . DIRECTORY_SEPARATOR,
        'color_list_folder' => _PS_MODULE_DIR_ . 'mxb_tools' . DIRECTORY_SEPARATOR . 'docs' . DIRECTORY_SEPARATOR,
        'color_list_filename' => 'html-colors-hex.csv',
    ];

    public static function getCsvHeaders($texture_array)
    {
        $array_keys = array_keys($texture_array[0]);

        return implode(';', $array_keys) . PHP_EOL;
    }

    public static function getTextureFormat()
    {
        return self::$definitions['texture_format'];
    }

    public static function importFromCsv($limit = 0, $id = 0, $tot_records = 0, $id_attribute_group = null)
    {
        // NO DEBUG: set $simulate to false !
        $simulate = false;
        $errors = [];
        $success = [];
        $limit = 0;

        $csv = MxbToolsUtils::csvToArray(
            self::$definitions['target_folder'] . self::$definitions['target_filename'],
            self::$definitions['csv_delimeter']
        );
        if (!$csv) {
            return [
                'records' => -1,
                'tot_records' => 0,
                'id' => 0,
                'errors' => MxbToolsTextureImport::addImportLog('-1', 'CSV FILE', 'error converting CSV file.'),
            ];
        }

        // check csv columns
        $res = self::validateCsvColumns($csv);
        if ($res !== true) {
            return [
                'records' => -1,
                'tot_records' => 0,
                'id' => 0,
                'errors' => MxbToolsTextureImport::addImportLog('-1', 'CSV FILE', $res),
            ];
        }

        // don't accept single record
        // if ($limit === 1)
        //  $limit = 2;

        // if ($limit > 0 && $id === 0) {
        if ((int) $tot_records === 0) {
            $tot_records = count($csv);
        }

        $languages = Language::getLanguages();
        $lang_iso_code = MxbToolsUtils::getConfigValue('MXB_TOOLS_IMPORT_COLORS_FIND_TEXTURE_LANG_ISO');
        $idLang = Language::getIdByIso($lang_iso_code);
        if (!$idLang) {
            $idLang = (int) Context::getContext()->language->id;
            $lang_iso_code = Language::getIsoById($idLang);
        }

        if (!$id_attribute_group) {
            $res = Db::getInstance()->executeS('
                SELECT DISTINCT agl.`name`, ag.*, agl.*
                FROM `' . _DB_PREFIX_ . 'attribute_group` ag
                ' . Shop::addSqlAssociation('attribute_group', 'ag') . '
                LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group_lang` agl
                    ON (ag.`id_attribute_group` = agl.`id_attribute_group` AND `id_lang` = ' . (int) $idLang . ')
                WHERE ag.is_color_group = 1
                ORDER BY `name` ASC
            ');
            if ($res) {
                $id_attribute_group = $res[0]['id_attribute_group'];
            }
        }

        if (!$id_attribute_group) {
            return [
                'records' => -1,
                'tot_records' => 0,
                'id' => 0,
                'errors' => MxbToolsTextureImport::addImportLog('-1', 'CSV FILE', 'no color attribute_group found.'),
            ];
        }
        $next_id = (int) $id + 1;
        $inserted_records = 0;

        // import main loop
        foreach ($csv as $rowkey => $row) {
            set_time_limit(300);

            // skip to current position
            if ((int) $rowkey < (int) $id) {
                continue;
            }
            $added = false;
            $next_id = (int) $rowkey + 1;

            /*
              CSV file must have every field with the right name:
                0 name
                1 filename
                2 hex_colors
                3-n: language iso codes (it, en ...)
            */
            $colorData = [];
            foreach ($row as $k => $c) {
                $colorData[$k] = $c;
            }
            $name = isset($colorData[$lang_iso_code]) ? $colorData[$lang_iso_code] : $colorData['name'];
            /*
            MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsTextureImport::importFromCsv',
                'LOOP ' . $rowkey . ' name='.$name.', ARRAY colorData=' . print_r($colorData, true),
                true
            );
            */
            if (empty($name)) {
                $errors[] = MxbToolsTextureImport::addImportLog($rowkey, '', 'missing color name !!!');

                // skip this color
                continue;
            }

            // look for existing color
            $attributes = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
                SELECT DISTINCT
                    a.id_attribute AS _id,
                    a.color,
                    a.id_attribute_group,
                    al.name AS name,
                    ag.is_color_group
                FROM `' . _DB_PREFIX_ . 'attribute` a
                LEFT JOIN `' . _DB_PREFIX_ . 'attribute_lang` al
                    ON (a.`id_attribute` = al.`id_attribute`
                    AND al.`id_lang` = ' . (int) $idLang . ')
                LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group` ag ON (a.`id_attribute_group` = ag.`id_attribute_group`)
                WHERE LOWER(al.name) = "' . Tools::strtolower($name) . '"
                AND a.id_attribute_group = ' . $id_attribute_group . '
                AND ag.is_color_group = 1
            ');
            $id_attribute = !$attributes ? 0 : (int) $attributes[0]['_id'];

            // create new attribute
            if ($id_attribute === 0) {
                $attribute = new Attribute();
                $attribute->id_attribute_group = $id_attribute_group;

                // set name in all languages
                $names = [];
                foreach ($languages as $lang) {
                    if ((string) $lang['iso_code'] !== (string) $lang_iso_code
                        && (isset($colorData[$lang['iso_code']]) && !empty($colorData[$lang['iso_code']]))) {
                        $names[$lang['id_lang']] = $colorData[$lang['iso_code']];
                    } else {
                        $names[$lang['id_lang']] = $name;
                    }
                }

                $attribute->name = $names;
                if ($colorData['hex_colors']) {
                    $hex = explode(',', $colorData['hex_colors']);
                    if (count($hex) === 1) {
                        $attribute->color = $colorData['hex_colors'];
                    }
                }

                if (!$simulate) {
                    if ($attribute->add()) {
                        $added = true;
                        /*
                        MxbToolsDebug::writeLog(
                            'Mxb_Tools',
                            'MxbToolsTextureImport::importFromCsv',
                            'LOOP ' . $rowkey . ' attribute='.$name.' added! - names='.print_r($names, true),
                            true
                        );
                        */
                        $success[] = MxbToolsTextureImport::addImportLog($rowkey, $name, 'ADDED.');

                        // import the texture
                        $res = self::saveColorTexture($attribute->id, $name, self::getDefaultTextureUrl());
                        if (count($res['errors'])) {
                            $errors = array_merge($errors, $res['errors']);
                        } elseif (count($res['success'])) {
                            $success = array_merge($success, $res['success']);
                        }
                    } else {
                        $errors[] = MxbToolsTextureImport::addImportLog(
                            $rowkey,
                            $colorData['name'],
                            'error adding attribute.'
                        );
                    }
                } else {
                    $added = true; // debug simulate
                }
            } else {
                // attribute exists
                $errors[] = MxbToolsTextureImport::addImportLog(
                    $rowkey,
                    $colorData['name'],
                    'attribute already exists.'
                );

                // import the texture
                $res = self::saveColorTexture(
                    $attributes[0]['_id'],
                    $attributes[0]['name'],
                    self::getDefaultTextureUrl()
                );
                if (count($res['errors'])) {
                    $errors = array_merge($errors, $res['errors']);
                } elseif (count($res['success'])) {
                    $success = array_merge($success, $res['success']);
                }
            }
            if ($added && $rowkey < count($csv) - 1) {
                ++$inserted_records;
            }
            if ($limit > 0 && $inserted_records >= $limit) {
                break;
            }
        } // foreach ($csv as $rowkey=>$row)

        $ret = [
            'records' => $inserted_records,
            'tot_records' => $tot_records,
            'id' => $next_id,
        ];
        if (count($errors)) {
            $ret['errors'] = str_replace('<br />', '', implode('|', $errors));
        }
        if (count($success)) {
            $ret['success'] = str_replace('<br />', '', implode('|', $success));
        }

        return $ret;
    }

    public static function addImportLog($rowkey = 0, $texture = '', $message = '')
    {
        return 'row ' . $rowkey . ' [' . $texture . '] ' . $message;
    }

    public static function uploadCsvFile($colors_list = null)
    {
        $source_fileinput = 'mxb_tools-upload-csv-texture';
        if (isset($_FILES[$source_fileinput])
            && isset($_FILES[$source_fileinput]['tmp_name'])
            && !empty($_FILES[$source_fileinput]['tmp_name'])
        ) {
            if (!move_uploaded_file(
                $_FILES[$source_fileinput]['tmp_name'],
                self::$definitions['target_folder'] . self::$definitions['target_filename']
            )) {
                return 'Error uploading CSV file';
            }

            // read new CSV file to elaborate it
            $csv = MxbToolsUtils::csvToArray(
                self::$definitions['target_folder'] . self::$definitions['target_filename'],
                self::$definitions['csv_delimeter']
            );

            // check csv columns
            $res = self::validateCsvColumns($csv);
            if ($res !== true) {
                return $res;
            }

            // check and generate missing hex
            $lang_iso_code_hex = MxbToolsUtils::getConfigValue('MXB_TOOLS_IMPORT_COLORS_FIND_HEX_LANG_ISO');
            $lang_iso_code_filename = MxbToolsUtils::getConfigValue('MXB_TOOLS_IMPORT_COLORS_FIND_TEXTURE_LANG_ISO');

            $edited = false;
            foreach ($csv as &$row) {
                // get color name for hex generation
                $color_name = isset($row[$lang_iso_code_filename]) ? $row[$lang_iso_code_filename] : $row['name'];
                if (!empty($color_name)) {
                    $row['filename'] = self::name2filename($color_name);
                    $edited = true;
                }
                if (!empty($row['filename']) && empty($row['hex_colors'])) {
                    // get color name for filename generation
                    $color_name = isset($row[$lang_iso_code_hex]) ? $row[$lang_iso_code_hex] : $row['name'];
                    $row['hex_colors'] = self::convertColorNameToHex($color_name, $colors_list);
                    /*
                    MxbToolsDebug::writeLog(
                        'mxb_tools',
                        'MxbToolsTextureImport::uploadCsvFile()',
                        'for ' . $row['name'] . ' hex is missing. Hex generated: ' . $row['hex_colors'],
                        true
                    );
                    */
                    $edited = true;
                }
            }

            // save CSV if edited
            if ($edited) {
                self::saveCsvFileFromArray(
                    $csv,
                    self::$definitions['target_folder'] . self::$definitions['target_filename']
                );
            }
            /*
            MxbToolsDebug::writeLog(
                'mxb_tools',
                'MxbToolsTextureImport::uploadCsvFile()',
                'csv: ' . print_r($csv, true),
                true
            );
            */
            // create image textures
            $res = self::createTextures($csv);
            if ($res === true) {
                return true;
            } elseif ($res === false) {
                return false;
            }

            return $res; // contains error message
        }

        return false;
    }

    public static function createTextures($csv)
    {
        if (!$csv) {
            return 'CSV invalid or empty';
        }
        if (!count($csv)) {
            return 'CSV invalid or empty';
        }

        // import main loop
        foreach ($csv as $row) {
            set_time_limit(300);
            /*
            MxbToolsDebug::writeLog(
                'mxb_tools',
                'MxbToolsTextureImport::createTextures()',
                'generating texture for ' . $row['filename'] . ', hex ' . $row['hex_colors'],
                true
            );
            */
            self::generateTextureThumb(
                $row['filename'],
                $row['hex_colors']
            );
        }

        return true;
    }

    public static function validateCsvColumns($csv)
    {
        foreach (self::$definitions['field_key'] as $key) {
            if (!array_key_exists($key, $csv[0])) {
                return 'CSV column ' . $key . ' does not exists';
            }
        }

        return true;
    }

    public static function generateTextureThumb($name, $hex_colors = null)
    {
        return MxbToolsUtils::generateImageByHexWeb(
            $hex_colors,
            self::$definitions['texture_folder'],
            $name,
            self::getTextureFormat()
        );
    }

    public static function importFromUrl($remote_addr = null)
    {
        $errors = [];
        $success = [];

        $remote_addr = self::validateTextureUrl($remote_addr);
        if ($remote_addr === false) {
            $errors[] = 'Please set the color textures remote address below and retry<br />';
            return ['errors' => $errors];
        }
        $lang_iso_code = MxbToolsUtils::getConfigValue('MXB_TOOLS_IMPORT_COLORS_FIND_TEXTURE_LANG_ISO');
        $attributes = self::getColorAttributes($lang_iso_code);
        if (empty($attributes)) {
            $errors[] = 'No color attributes compatible with existing textures found.<br />';
            return ['errors' => $errors];
        }

        foreach ($attributes as $attribute) {
            $name = isset($attribute['name_lang'][$lang_iso_code])
                ? $attribute['name_lang'][$lang_iso_code]
                : $attribute['name'];
            $res = self::saveColorTexture($attribute['_id'], $name, $remote_addr);
            /*
             MxbToolsDebug::writeLog(
                'Mxb_Tools',
                'MxbToolsTextureImport::importFromUrl',
                'saveColorTexture res=' . print_r($res, true),
                true
            );
            */
            if (count($res['errors'])) {
                $errors = array_merge($errors, $res['errors']);
            } elseif (count($res['success'])) {
                $success = array_merge($success, $res['success']);
            }
        }

        return [
            'errors' => $errors,
            'success' => $success,
        ];
    }

    public static function getColorAttributes($lang_iso_code = null)
    {
        if (!$lang_iso_code) {
            $lang_iso_code = MxbToolsUtils::getConfigValue('MXB_TOOLS_IMPORT_COLORS_FIND_HEX_LANG_ISO');
        }
        $idLang = Language::getIdByIso($lang_iso_code);
        if (!$idLang) {
            $idLang = (int) Context::getContext()->language->id;
        }

        $attributes = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
            SELECT DISTINCT
                a.id_attribute AS _id,
                al.name AS name
            FROM `' . _DB_PREFIX_ . 'attribute` a
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_lang` al
                ON (a.`id_attribute` = al.`id_attribute`
                AND al.`id_lang` = ' . $idLang . ')
            LEFT JOIN `' . _DB_PREFIX_ . 'attribute_group` ag ON (a.`id_attribute_group` = ag.`id_attribute_group`)
            WHERE LENGTH(a.color) = 0
            AND LENGTH(al.name) > 0
            AND ag.is_color_group = 1
        ');
        // load names in various languages
        foreach ($attributes as &$att) {
            $names = Db::getInstance(_PS_USE_SQL_SLAVE_)->executeS('
                SELECT *
                FROM `' . _DB_PREFIX_ . 'attribute_lang`
                WHERE `id_attribute` = ' . $att['_id'] . '
                ORDER BY id_lang
            ');
            $att['name_lang'] = [];
            foreach ($names as &$n) {
                $n['iso_code'] = Language::getIsoById($n['id_lang']);
                $att['name_lang'][$n['iso_code']] = $n['name'];
            }
            // $att[$n['iso_code']] = $n['name'];
        }

        return $attributes;
    }

    public static function convertColorNameToHex($name, $colors_list = null)
    {
        if (empty($colors_list)) {
            return '';
        }
        $langCode = MxbToolsUtils::getConfigValue('MXB_TOOLS_IMPORT_COLORS_FIND_HEX_LANG_ISO');
        $names = self::name2array($name);
        $colors_hex = [];
        if (!array_key_exists($langCode, $colors_list[0])) {
            $langCode = 'en';
        }
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsTextureImport::convertColorNameToHex',
            'name=' . $name . ', langCode=' . $langCode . ' names=' . print_r($names, true),
            true
        );
        */
        foreach ($names as $color_name) {
            foreach ($colors_list as $list) {
                if (self::cleanName($list[$langCode]) == $color_name) {
                    $colors_hex[] = $list['hex'];
                    break;
                }
            }
        }
        /*
        MxbToolsDebug::writeLog(
            'Mxb_Tools',
            'MxbToolsTextureImport::convertColorNameToHex',
            'color name=' . $name . ', colors_hex=' . print_r($colors_hex, true),
            true
        );
        */
        if (count($colors_hex)) {
            return implode(',', $colors_hex);
        }

        return '';
    }

    public static function getColorAttributeList($colors_list = null)
    {
        if ($colors_list === null) {
            // load color list for name to hex conversion
            $colors_list = MxbToolsUtils::csvToArray(
                self::$definitions['color_list_folder'] . self::$definitions['color_list_filename'],
                self::$definitions['csv_delimeter']
            );
        }

        // load color textures
        $textures = [];
        $attributes = self::getColorAttributes(
            MxbToolsUtils::getConfigValue('MXB_TOOLS_IMPORT_COLORS_FIND_HEX_LANG_ISO')
        );
        if ($attributes && count($attributes)) {
            foreach ($attributes as $attribute) {
                $fname = self::name2filename($attribute['name']);
                $hex_colors = (!empty($colors_list)
                    ? self::convertColorNameToHex($attribute['name'], $colors_list)
                    : '');
                $row = [
                    self::$definitions['field_key']['name'] => trim($attribute['name']),
                    self::$definitions['field_key']['filename'] => $fname,
                    self::$definitions['field_key']['hex_colors'] => $hex_colors,
                ];
                foreach ($attribute['name_lang'] as $keyatt => $att) {
                    $row[$keyatt] = $att;
                }
                $textures[] = $row;
            }
            // save default texture csv list
            self::saveCsvFileFromArray(
                $textures,
                self::$definitions['target_folder'] . self::$definitions['sample_filename']
            );
        }

        return $textures;
    }

    public static function saveCsvFileFromArray($texture_array, $target_filename = null)
    {
        $records = self::getCsvHeaders($texture_array);
        foreach ($texture_array as $row) {
            $records .= implode(';', $row) . PHP_EOL;
            // $records .= trim($row['name']) . ';' . $row['filename'] . ';' . $row['hex_colors'] . PHP_EOL;
        }

        // create CSV file
        MxbToolsUtils::saveCsvFile(
            $target_filename,
            $records
        );
    }

    public static function saveColorTexture($id_attribute, $name, $remote_addr = null)
    {
        $success = [];
        $errors = [];
        $remote_addr = self::validateTextureUrl($remote_addr);
        if ($remote_addr === false) {
            return false;
        }

        // $name = strtolower(trim($attribute['name']));
        $file = $remote_addr . self::name2filename($name);
        $ext = '.' . MxbToolsTextureImport::getTextureFormat();
        $newfile = _PS_IMG_DIR_ . 'co/' . $id_attribute . $ext;
        try {
            if (copy(str_replace(' ', '%20', $file), $newfile)) {
                $success[] = $name . ' texture copied as ' . $newfile . '<br />';
            } else {
                $errors[] = $name . ': error copying texture (does '
                    . $name . $ext . ' exists in remote folder ?)<br />';
            }
        } catch (Exception $e) {
            $errors[] = $name . ': error copying texture (does '
                . $name . $ext . ' exists in remote folder ?) Error is: ' . $e->getMessage() . '<br />';
        }

        return [
            'errors' => $errors,
            'success' => $success,
        ];
    }

    public static function validateTextureUrl($remote_addr = null)
    {
        if ($remote_addr === false || $remote_addr === null) {
            $remote_addr = MxbToolsUtils::getConfigValue('MXB_TOOLS_COLOR_TEXTURES_REMOTE_ADDRESS');
        }
        if ($remote_addr === false || $remote_addr == '') {
            return false;
        }
        if (Tools::substr($remote_addr, -1) != '/') {
            $remote_addr .= '/';
        }

        return $remote_addr;
    }

    public static function getDefaultTextureUrl()
    {
        return MxbToolsUtils::path2url(self::$definitions['texture_folder']);
    }

    public static function cleanName($name)
    {
        return Tools::strtolower(str_replace(' ', '', trim($name)));
    }

    public static function name2array($name)
    {
        if (strpos($name, ',') !== false) {
            return explode(',', self::cleanName($name));
        } elseif (strpos($name, '-') !== false) {
            return explode('-', self::cleanName($name));
        } elseif (strpos($name, '/') !== false) {
            return explode('/', self::cleanName($name));
        } elseif (strpos($name, '|') !== false) {
            return explode('|', self::cleanName($name));
        } elseif (strpos($name, '&') !== false) {
            return explode('&', self::cleanName($name));
        } else {
            $name = Tools::strtolower($name);
            if (strpos($name, ' and ') !== false) {
                $haystack = ' and ';
            } elseif (strpos($name, ' e ') !== false) {
                $haystack = ' e ';
            } elseif (strpos($name, ' y ') !== false) {
                $haystack = ' y ';
            } elseif (strpos($name, ' und ') !== false) {
                $haystack = ' und ';
            } else {
                return [self::cleanName($name)];
            }
            $arr = explode($haystack, $name);
            foreach ($arr as &$a) {
                $a = self::cleanName($a);
            }

            return $arr;
        }
    }

    public static function name2filename($name)
    {
        return MxbToolsUtils::normalizeFilename(
            implode(',', self::name2array($name))
        ) . '.' . self::getTextureFormat();
    }
}
